r"""

  _ __ ___   _ __  _   _
 | '__/ _ \ | '_ \| | | |
 | | | (_) || |_) | |_| |
 |_|  \___(_) .__/ \__, |
            | |     __/ |
            |_|    |___/

ro.py
by jmkdev

"""

from ro_py.users import User
from ro_py.groups import Group
from ro_py.badges import Badge
from ro_py.games import Game, game_from_place_id
from ro_py.assets import Asset



"""

ro.py > assets.py

This file houses functions and classes that pertain to Roblox assets.

"""

from ro_py import User, Group
from ro_py.errors import NotLimitedError
from ro_py.asset_type import asset_types
import iso8601
import ro_py.ro_py_requests as requests

endpoint = "https://api.roblox.com/"


class LimitedResaleData:
    """
    Represents the resale data of a limited item.
    """
    def __init__(self, resale_data):
        self.asset_stock = resale_data["assetStock"]
        self.sales = resale_data["sales"]
        self.number_remaining = resale_data["numberRemaining"]
        self.recent_average_price = resale_data["recentAveragePrice"]
        self.original_price = resale_data["originalPrice"]


class Asset:
    """
    Represents an asset.
    """
    def __init__(self, asset_id):
        asset_info_req = requests.get(
            url=endpoint + "marketplace/productinfo",
            params={
                "assetId": asset_id
            }
        )
        asset_info = asset_info_req.json()
        self.target_id = asset_info["TargetId"]
        self.product_type = asset_info["ProductType"]
        self.asset_id = asset_info["AssetId"]
        self.product_id = asset_info["ProductId"]
        self.name = asset_info["Name"]
        self.description = asset_info["Description"]
        self.asset_type_id = asset_info["AssetTypeId"]
        self.asset_type_name = asset_types[self.asset_type_id]
        if asset_info["Creator"]["CreatorType"] == "User":
            self.creator = User(asset_info["Creator"]["Id"])
        elif asset_info["Creator"]["CreatorType"] == "Group":
            self.creator = Group(asset_info["Creator"]["CreatorTargetId"])
        self.created = iso8601.parse_date(asset_info["Created"])
        self.updated = iso8601.parse_date(asset_info["Updated"])
        self.price = asset_info["PriceInRobux"]
        self.is_new = asset_info["IsNew"]
        self.is_for_sale = asset_info["IsForSale"]
        self.is_public_domain = asset_info["IsPublicDomain"]
        self.is_limited = asset_info["IsLimited"]
        self.is_limited_unique = asset_info["IsLimitedUnique"]
        self.minimum_membership_level = asset_info["MinimumMembershipLevel"]
        self.content_rating_type_id = asset_info["ContentRatingTypeId"]

    @property
    def remaining(self):
        asset_info_req = requests.get(
            url=endpoint + "marketplace/productinfo",
            params={
                "assetId": self.asset_id
            }
        )
        asset_info = asset_info_req.json()
        return asset_info["Remaining"]

    @property
    def limited_resale_data(self):
        if self.is_limited:
            resale_data_req = requests.get(f"https://economy.roblox.com/v1/assets/{self.asset_id}/resale-data")
            return LimitedResaleData(resale_data_req.json())
        else:
            raise NotLimitedError("You can only read this information on limited items.")



"""

ro.py > asset_type.py

This file is a conversion table for asset type IDs to asset type names.

"""

asset_types = [
    None,
    "Image",
    "TeeShirt",
    "Audio",
    "Mesh",
    "Lua",
    "Hat",
    "Place",
    "Model",
    "Shirt",
    "Pants",
    "Decal",
    "Head",
    "Face",
    "Gear",
    "Badge",
    "Animation",
    "Torso",
    "RightArm",
    "LeftArm",
    "LeftLeg",
    "RightLeg",
    "Package",
    "GamePass",
    "Plugin",
    "MeshPart",
    "HairAccessory",
    "FaceAccessory",
    "NeckAccessory",
    "ShoulderAccessory",
    "FrontAccesory",
    "BackAccessory",
    "WaistAccessory",
    "ClimbAnimation",
    "DeathAnimation",
    "FallAnimation",
    "IdleAnimation",
    "JumpAnimation",
    "RunAnimation",
    "SwimAnimation",
    "WalkAnimation",
    "PoseAnimation",
    "EarAccessory",
    "EyeAccessory",
    "EmoteAnimation",
    "Video"
]



"""

ro.py > users.py

This file houses functions and classes that pertain to Roblox users and profiles.

"""

from ro_py.robloxbadges import RobloxBadge
import ro_py.ro_py_requests as requests
import iso8601

endpoint = "https://users.roblox.com/"


class User:
    """
    Represents a Roblox user and their profile.
    Can be initialized with either a user ID or a username.
    """
    def __init__(self, ui):
        if isinstance(ui, str):
            is_id = False
            try:
                int(str)
                is_id = True
            except TypeError:
                is_id = False
            if is_id:
                self.id = int(ui)
            else:
                user_id_req = requests.post(
                    url="https://users.roblox.com/v1/usernames/users",
                    json={
                        "usernames": [
                            ui
                        ]
                    }
                )
                user_id = user_id_req.json()["data"][0]["id"]
                self.id = user_id
        elif isinstance(ui, int):
            self.id = ui

        user_info_req = requests.get(endpoint + f"v1/users/{self.id}")
        user_info = user_info_req.json()
        self.description = user_info["description"]
        self.created = iso8601.parse_date(user_info["created"])
        self.is_banned = user_info["isBanned"]
        self.name = user_info["name"]
        self.display_name = user_info["displayName"]
        # has_premium_req = requests.get(f"https://premiumfeatures.roblox.com/v1/users/{self.id}/validate-membership")
        # self.has_premium = has_premium_req

    @property
    def status(self):
        status_req = requests.get(endpoint + f"v1/users/{self.id}/status")
        return status_req.json()["status"]

    @property
    def roblox_badges(self):
        roblox_badges_req = requests.get(f"https://accountinformation.roblox.com/v1/users/{self.id}/roblox-badges")
        roblox_badges = []
        for roblox_badge_data in roblox_badges_req.json():
            roblox_badges.append(RobloxBadge(roblox_badge_data))
        return roblox_badges



"""

ro.py > groups.py

This file houses functions and classes that pertain to Roblox groups.

"""

from ro_py import User, thumbnails
import ro_py.ro_py_requests as requests

endpoint = "https://groups.roblox.com/"


class Shout:
    """
    Represents a group shout.
    """
    def __init__(self, shout_data):
        self.body = shout_data["body"]
        self.poster = User(shout_data["poster"]["userId"])


class Group:
    """
    Represents a group.
    """
    def __init__(self, group_id):
        self.id = group_id
        group_info_req = requests.get(endpoint + f"v1/groups/{self.id}")
        group_info = group_info_req.json()
        self.name = group_info["name"]
        self.description = group_info["description"]
        self.owner = User(group_info["owner"]["userId"])

        self.member_count = group_info["memberCount"]
        self.is_builders_club_only = group_info["isBuildersClubOnly"]
        self.public_entry_allowed = group_info["publicEntryAllowed"]
        # self.is_locked = group_info["isLocked"]

    @property
    def shout(self):
        """
        :return: An instance of Shout
        """
        group_info_req = requests.get(endpoint + f"v1/groups/{self.id}")
        group_info = group_info_req.json()

        if group_info["shout"]:
            return Shout(group_info["shout"])
        else:
            return None

    def get_icon(self, size=thumbnails.size_150x150, format=thumbnails.format_png, is_circular=False):
        """
        Equivalent to thumbnails.get_group_icon
        """
        return thumbnails.get_group_icon(self, size, format, is_circular)



"""

ro.py > ro_py_requests.py

This file houses functions and classes that pertain to web requests.
It is essentially a very limited Roblox-specific version of requests.

"""

from ro_py.errors import ApiError
import requests


def get(*args, **kwargs):
    get_request = requests.get(*args, **kwargs)
    try:
        get_request_error = get_request.json()["errors"]
    except KeyError:
        return get_request

    raise ApiError(get_request_error[0]["message"])


def post(*args, **kwargs):
    post_request = requests.post(*args, **kwargs)
    try:
        post_request_error = post_request.json()["errors"]
    except KeyError:
        return post_request

    raise ApiError(post_request_error[0]["message"])




"""

ro.py > robloxbadges.py

This file houses functions and classes that pertain to roblox-awarded badges.

"""


class RobloxBadge:
    """
    Represents a Roblox badge.
    This is not equivalent to a badge you would earn from a game.
    This class represents a Roblox-awarded badge as seen in https://www.roblox.com/info/roblox-badges.
    """
    def __init__(self, roblox_badge_data):
        self.id = roblox_badge_data["id"]
        self.name = roblox_badge_data["name"]
        self.description = roblox_badge_data["description"]
        self.image_url = roblox_badge_data["imageUrl"]



"""

ro.py > badges.py

This file houses functions and classes that pertain to game-awarded badges.

"""

import ro_py.ro_py_requests as requests

endpoint = "https://badges.roblox.com/"


class BadgeStatistics:
    """
    Represents a badge's statistics.
    """
    def __init__(self, past_date_awarded_count, awarded_count, win_rate_percentage):
        self.past_date_awarded_count = past_date_awarded_count
        self.awarded_count = awarded_count
        self.win_rate_percentage = win_rate_percentage


class Badge:
    """
    Represents a game-awarded badge.
    """
    def __init__(self, badge_id):
        self.id = badge_id
        badge_info_req = requests.get(endpoint + f"v1/badges/{badge_id}")
        badge_info = badge_info_req.json()
        self.name = badge_info["name"]
        self.description = badge_info["description"]
        self.display_name = badge_info["displayName"]
        self.display_description = badge_info["displayDescription"]
        self.enabled = badge_info["enabled"]
        statistics_info = badge_info["statistics"]
        self.statistics = BadgeStatistics(
            statistics_info["pastDayAwardedCount"],
            statistics_info["awardedCount"],
            statistics_info["winRatePercentage"]
        )



"""

ro.py > games.py

This file houses functions and classes that pertain to Roblox universes and places.

"""

from ro_py import User, Group, Badge, thumbnails
import ro_py.ro_py_requests as requests

endpoint = "https://games.roblox.com/"


class Votes:
    """
    Represents a game's votes.
    """
    def __init__(self, votes_data):
        self.up_votes = votes_data["upVotes"]
        self.down_votes = votes_data["downVotes"]


class Game:
    """
    Represents a Roblox game universe.
    This class represents multiple game-related endpoints.
    """
    def __init__(self, universe_id):
        self.id = universe_id
        game_info_req = requests.get(
            url=endpoint + "v1/games",
            params={
                "universeIds": str(self.id)
            }
        )
        game_info = game_info_req.json()
        game_info = game_info["data"][0]
        self.name = game_info["name"]
        self.description = game_info["description"]
        if game_info["creator"]["type"] == "User":
            self.creator = User(game_info["creator"]["id"])
        elif game_info["creator"]["type"] == "Group":
            self.creator = Group(game_info["creator"]["id"])
        self.price = game_info["price"]
        self.allowed_gear_genres = game_info["allowedGearGenres"]
        self.allowed_gear_categories = game_info["allowedGearCategories"]
        self.max_players = game_info["maxPlayers"]
        self.studio_access_to_apis_allowed = game_info["studioAccessToApisAllowed"]
        self.create_vip_servers_allowed = game_info["createVipServersAllowed"]
        self.__cached_badges = False

    @property
    def votes(self):
        """
        :return: An instance of Votes
        """
        votes_info_req = requests.get(
            url=endpoint + "v1/games/votes",
            params={
                "universeIds": str(self.id)
            }
        )
        votes_info = votes_info_req.json()
        votes_info = votes_info["data"][0]
        votes = Votes(votes_info)
        return votes

    def get_icon(self, size=thumbnails.size_256x256, format=thumbnails.format_png, is_circular=False):
        """
        Equivalent to thumbnails.get_game_icon
        """
        return thumbnails.get_game_icon(self, size, format, is_circular)

    @property
    def badges(self):
        """
        Note: this has a limit of 100 badges due to paging. This will be expanded soon.
        :return: A list of Badge instances
        """
        if not self.__cached_badges:
            badges_req = requests.get(
                url=f"https://badges.roblox.com/v1/universes/{self.id}/badges",
                params={
                    "limit": 100,
                    "sortOrder": "Asc"
                }
            )
            badges_data = badges_req.json()["data"]
            badges = []
            for badge in badges_data:
                badges.append(Badge(badge["id"]))
            self.__cached_badges = badges

        return self.__cached_badges


def place_id_to_universe_id(place_id):
    """
    Returns the containing universe ID of a place ID.
    :param place_id: Place ID
    :return: Universe ID
    """
    universe_id_req = requests.get(
        url="https://api.roblox.com/universes/get-universe-containing-place",
        params={
            "placeId": place_id
        }
    )
    universe_id = universe_id_req.json()["UniverseId"]
    return universe_id


def game_from_place_id(place_id):
    """
    Generates an instance of Game with a place ID instead of a game ID.
    :param place_id: Place ID
    :return: Instace of Game
    """
    return Game(place_id_to_universe_id(place_id))



"""

ro.py > thumbnails.py

This file houses functions and classes that pertain to Roblox icons and thumbnails.

"""

from ro_py.errors import InvalidShotTypeError
import ro_py.ro_py_requests as requests

endpoint = "https://thumbnails.roblox.com/"

PlaceHolder = "PlaceHolder"
AutoGenerated = "AutoGenerated"
ForceAutoGenerated = "ForceAutoGenerated"

AvatarFullBody = 0
AvatarBust = 1
AvatarHeadshot = 2

size_30x30 = "30x30"
size_42x42 = "42x42"
size_48x48 = "48x48"
size_50x50 = "50x50"
size_60x62 = "60x62"
size_75x75 = "75x75"
size_110x110 = "110x110"
size_128x128 = "128x128"
size_140x140 = "140x140"
size_150x150 = "150x150"
size_160x100 = "160x100"
size_250x250 = "250x250"
size_256x144 = "256x144"
size_256x256 = "256x256"
size_300x250 = "300x240"
size_304x166 = "304x166"
size_384x216 = "384x216"
size_396x216 = "396x216"
size_420x420 = "420x420"
size_480x270 = "480x270"
size_512x512 = "512x512"
size_576x324 = "576x324"
size_720x720 = "720x720"
size_768x432 = "768x432"

format_png = "Png"
format_jpg = "Jpeg"
format_jpeg = "Jpeg"


def get_group_icon(group, size=size_150x150, format=format_png, is_circular=False):
    """
    Gets a game's icon.
    :param group: The group.
    :param size: The thumbnail size, formatted widthxheight.
    :param format: The thumbnail format
    :param is_circular: The circle thumbnail output parameter.
    :return: Image URL
    """
    group_icon_req = requests.get(
        url=endpoint + "v1/groups/icons",
        params={
            "groupIds": str(group.id),
            "size": size,
            "format": format,
            "isCircular": is_circular
        }
    )
    group_icon = group_icon_req.json()["data"][0]["imageUrl"]
    return group_icon


def get_game_icon(game, size=size_256x256, format=format_png, is_circular=False):
    """
    Gets a game's icon.
    :param game: The game.
    :param size: The thumbnail size, formatted widthxheight.
    :param format: The thumbnail format
    :param is_circular: The circle thumbnail output parameter.
    :return: Image URL
    """
    game_icon_req = requests.get(
        url=endpoint + "v1/games/icons",
        params={
            "universeIds": str(game.id),
            "returnPolicy": PlaceHolder,
            "size": size,
            "format": format,
            "isCircular": is_circular
        }
    )
    game_icon = game_icon_req.json()["data"][0]["imageUrl"]
    return game_icon


def get_avatar_image(user, shot_type=AvatarFullBody, size=None, format=format_png, is_circular=False):
    """
    Gets a full body, bust, or headshot image of a user.
    :param user: User to use for avatar.
    :param shot_type: Type of shot.
    :param size: The thumbnail size, formatted widthxheight.
    :param format: The thumbnail format
    :param is_circular: The circle thumbnail output parameter.
    :return: Image URL
    """
    shot_endpoint = endpoint + "v1/users/"
    if shot_type == AvatarFullBody:
        shot_endpoint = shot_endpoint + "avatar"
        size = size or size_30x30
    elif shot_type == AvatarBust:
        shot_endpoint = shot_endpoint + "avatar-bust"
        size = size or size_50x50
    elif shot_type == AvatarHeadshot:
        size = size or size_48x48
        shot_endpoint = shot_endpoint + "avatar-headshot"
    else:
        raise InvalidShotTypeError("Invalid shot type.")
    shot_req = requests.get(
        url=shot_endpoint,
        params={
            "userIds": str(user.id),
            "size": size,
            "format": format,
            "isCircular": is_circular
        }
    )
    return shot_req.json()["data"][0]["imageUrl"]



"""

ro.py > errors.py

This file houses custom exceptions unique to this module.

"""


class NotLimitedError(Exception):
    """Called when code attempts to read limited-only information."""
    pass


class InvalidIconSizeError(Exception):
    """Called when code attempts to pass in an improper size to a thumbnail function."""
    pass


class InvalidShotTypeError(Exception):
    """Called when code attempts to pass in an improper avatar image type to a thumbnail function."""
    pass


class ApiError(Exception):
    """Called in ro_py_requets when an API request fails."""
    pass
